/***************************************************************************
                          calltreedlg.cpp
                             -------------------
    begin                : Fri Jul 18 2003
    copyright            : (C) 2003 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <qapplication.h>
#include <qclipboard.h>
#include <qfile.h>
#include <qtoolbutton.h>
#include <klocale.h>
#include <kfiledialog.h>
#include "calltreedlg.h"
#include "cscopefrontend.h"
#include "queryresultsmenu.h"
#include "quickdefdlg.h"
#include "graphwidget.h"
#include "kscopepixmaps.h"

/** File Name index for the file name generation */
int CallTreeDlg::s_nFileNameIndex = 0;

/**
 * Class constructor.
 * @param	pParent	The parent widget
 * @param	szName	The widget's name
 */
CallTreeDlg::CallTreeDlg(QWidget* pParent, const char* szName) :
	CallTreeLayout(pParent, szName),
	m_progress(m_pGraphWidget)
{
	// Make sure the dialog object is deleted when the dialog is closed
	setWFlags(getWFlags() | WDestructiveClose);

	// Set button pixmaps
	m_pSaveButton->setPixmap(GET_PIXMAP(ButtonSaveAs));
	m_pZoomInButton->setPixmap(GET_PIXMAP(ButtonZoomIn));
	m_pZoomOutButton->setPixmap(GET_PIXMAP(ButtonZoomOut));
	m_pRotateButton->setPixmap(GET_PIXMAP(ButtonRotate));
	
	// Create a persistent Cscope process
	m_pCscope = new CscopeFrontend();
	
	// Add records output by the Cscope process
	connect(m_pCscope, SIGNAL(dataReady(FrontendToken*)), this,
		SLOT(slotDataReady(FrontendToken*)));
	
	// Display query progress information
	connect(m_pCscope, SIGNAL(progress(int, int)), this,
		SLOT(slotProgress(int, int)));	
		
	// Makes an item child-less if the query returns no results
	connect(m_pCscope, SIGNAL(finished(uint)), this,
		SLOT(slotFinished(uint)));
	
	// Show called functions
	connect(m_pGraphWidget, SIGNAL(showCalled(const QString&)), this, 
		SLOT(slotQueryCalled(const QString&)));
	
	// Show calling functions
	connect(m_pGraphWidget, SIGNAL(showCalling(const QString&)), this, 
		SLOT(slotQueryCalling(const QString&)));
	
	// Find the definition of a function
	connect(m_pGraphWidget, SIGNAL(findDef(const QString&)), this, 
		SLOT(slotFindDef(const QString&)));
	
	// Open the location of a call
	connect(m_pGraphWidget, SIGNAL(lineRequested(const QString&, uint)),
		this, SIGNAL(lineRequested(const QString&, uint)));
}

/**
 * Class destructor.
 */
CallTreeDlg::~CallTreeDlg()
{
}

/**
 * @param	sFunc	The function to use as the root of the call tree
 */
void CallTreeDlg::setRoot(const QString& sFunc)
{
	// Generate unique file name to save call tree later
	m_sFileName = sFunc;
	m_sFileName.replace(' ', '_');
	m_sFileName += QString::number(++s_nFileNameIndex);

	// Create an initial node
	m_pGraphWidget->setRoot(sFunc);
	m_pGraphWidget->draw();
}

/**
 * Informs the call tree manager that this object should be removed from the
 * list of call tree dialogues.
 * @param	pEvent	Information on the closing event 
 */
void CallTreeDlg::closeEvent(QCloseEvent* pEvent)
{
	emit closed(this);
	QDialog::closeEvent(pEvent);
}

/**
 * Adds an entry to the tree, as the child of the active item.
 * Called by a CscopeFrontend object, when a new entry was received in its
 * whole from the Cscope back-end process. The entry contains the data of a
 * function calling the function described by the active item.
 * @param	pToken	The first token in the entry
 */
void CallTreeDlg::slotDataReady(FrontendToken* pToken)
{
	GraphWidget::CallData data;
	QString sFunc;
	
	// Get the file name
	data.m_sFile = pToken->getData();
	pToken = pToken->getNext();

	// Get the function name
	sFunc = pToken->getData();
	pToken = pToken->getNext();

	// Get the line number (do not accept global information on a call tree)
	data.m_sLine = pToken->getData();
	if (data.m_sLine.toUInt() == 0)
		return;
		
	pToken = pToken->getNext();

	// Get the line's text
	data.m_sText = pToken->getData();
		
	// Determine the caller and the callee
	if (m_bCalled) {
		data.m_sCaller = m_sQueriedFunc;
		data.m_sCallee = sFunc;
	}
	else {
		data.m_sCaller = sFunc;
		data.m_sCallee = m_sQueriedFunc;
	}
		
	// Add the call to the graph
	m_pGraphWidget->addCall(data);
}

/**
 * Displays search progress information.
 * This slot is connected to the progress() signal emitted by a
 * CscopeFrontend object.
 * @param	nProgress	The current progress value
 * @param	nTotal		The expected final value
 */
void CallTreeDlg::slotProgress(int nProgress, int nTotal)
{
	m_progress.setProgress(nProgress, nTotal);
}

/**
 * Disables the expandability feature of an item, if no functions calling it
 * were found.
 * @param	nRecords	Number of records reported by the query
 */
void CallTreeDlg::slotFinished(uint /*nRecords*/)
{
	// Destroy the progress bar
	m_progress.finished();
		
	// Redraw the graph
	m_pGraphWidget->draw();
}

/**
 * Queries Cscope for functions calling the specified function.
 * @param	sFunc	Function name to query
 */
void CallTreeDlg::query(const QString& sFunc, bool bCalled)
{
	m_sQueriedFunc = sFunc;
	m_bCalled = bCalled;
	
	if (bCalled)
		m_pCscope->query(CscopeFrontend::Called, sFunc);
	else
		m_pCscope->query(CscopeFrontend::Calling, sFunc);
}

/**
 * Restores a call tree from the given call tree file.
 * NOTE: The call tree file is deleted when loading is complete.
 * @param	sProjPath	The full path of the project directory
 * @param	sFileName	The name of the call tree file to load
 * @return	true if successful, false otherwise
 */
bool CallTreeDlg::load(const QString& sProjPath, const QString& sFileName)
{
	QString sPath;
	
	// Load the graph from the given file
	sPath = sProjPath + "/" + sFileName;
	if (!m_pGraphWidget->load(sPath))
		return false;
	
	// Store the file name
	m_sFileName = sFileName;
	
	// Draw the graph
	m_pGraphWidget->draw();
	return true;
}

/**
 * Writes the contents of the call tree dialog to a call tree file.
 * This method is called for call trees before the owner project is
 * closed.
 * @param	sProjPath	The full path of the project directory
 */
void CallTreeDlg::store(const QString& sProjPath)
{
	QString sPath;
	
	sPath = sProjPath + "/" + m_sFileName;
	m_pGraphWidget->save(sPath, true);
}

/**
 * Saves the graph to a dot file.
 * The user is prompted for a name to use for the file, and then graph
 * widget writes its information to this file (using the dot language).
 * This slot is connected to the clicked() signal of the "Save As..." button.
 */
void CallTreeDlg::slotSaveClicked()
{
	QString sFile;
	
	// Prompt the user for a file name
	sFile = KFileDialog::getSaveFileName(":kscope");
	
	// Save the graph to a file (unless the user did not give a file name)
	if (!sFile.isEmpty())
		m_pGraphWidget->save(sFile, false);
}

/**
 * Increases the zoom factor of the graph.
 * This slot is connected to the clicked() signal of the "Zoom In" button.
 */
void CallTreeDlg::slotZoomInClicked()
{
	m_pGraphWidget->zoom(true);
	m_pGraphWidget->draw();
}

/**
 * Decreases the zoom factor of the graph.
 * This slot is connected to the clicked() signal of the "Zoom Out" button.
 */
void CallTreeDlg::slotZoomOutClicked()
{
	m_pGraphWidget->zoom(false);
	m_pGraphWidget->draw();
}

/**
 * Changes the graph's layout direction.
 * This slot is connected to the clicked() signal of the "Rotate" button.
 */
void CallTreeDlg::slotRotateClicked()
{
	m_pGraphWidget->rotate();
	m_pGraphWidget->draw();
}

/**
 * Finds all functions called from the given function.
 * This slot is connected to the queryCalled() signal of the node popup menu.
 * @param	sFunc	The function to query
 */
void CallTreeDlg::slotQueryCalled(const QString& sFunc)
{
	query(sFunc, true);
}

/**
 * Finds all functions calling to the given function.
 * This slot is connected to the queryCalling() signal of the node popup menu.
 * @param	sFunc	The function to query
 */
void CallTreeDlg::slotQueryCalling(const QString& sFunc)
{
	query(sFunc, false);
}

/**
 * Starts a quick definition process for the given function.
 * This slot is connected to the findDef() signal of the node popup menu.
 * @param	sFunc	The function to query
 */
void CallTreeDlg::slotFindDef(const QString& sFunc)
{
	QuickDefDlg* pDlg;
		 
	pDlg = new QuickDefDlg(this);
	connect(pDlg, SIGNAL(lineRequested(const QString&, uint)), this,
		SIGNAL(lineRequested(const QString&, uint)));
		
	pDlg->query(sFunc);
}

#include "calltreedlg.moc"
