/***************************************************************************
                             quickdefdlg.cpp
                             -------------------
    begin                : Tue Jun 14 2005
    copyright            : (C) 2005 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <klocale.h>
#include "quickdefdlg.h"

#include <kdebug.h>

/**
 * Class Constructor.
 * @param	pParent	The parent widget
 * @param	szName	The widget's name
 */
QuickDefDlg::QuickDefDlg(QWidget* pParent, const char* szName) :
	QuickDefLayout(pParent, szName, false, 0),
	m_progress(m_pList)
{
}

/**
 * Class destructor.
 */
QuickDefDlg::~QuickDefDlg()
{
}

/**
 * Starts a Cscope query that looks for the definition of the given
 * function name.
 * @param	sFunc	The function whose definition is requested
 */
void QuickDefDlg::query(const QString& sFunc)
{
	CscopeFrontend* pCscope;
	
	// Set the dialogue's caption
	setCaption(i18n("Definition of ") + sFunc);
	
	// Create a self-destructing Cscope process
	pCscope = new CscopeFrontend(true);
	
	// Handle signals emitted by the process
	connect(pCscope, SIGNAL(dataReady(FrontendToken*)), this,
		SLOT(slotDataReady(FrontendToken*)));
	connect(pCscope, SIGNAL(progress(int, int)), this,
		SLOT(slotProgress(int, int)));
	connect(pCscope, SIGNAL(finished(uint)), this, SLOT(slotFinished(uint)));
	
	// Query the function's definition
	pCscope->query(CscopeFrontend::Definition, sFunc);
}

/**
 * Emits the lineRequested() signal when a list view item is selected, and
 * destroys the dialogue.
 * This slot is connected to both the list view's doubleClicked() and its
 * returnPressed() signals.
 * @param	pItem	The selected item
 */
void QuickDefDlg::slotRecordSelected(QListViewItem* pItem)
{
	uint nLine;
	
	// Ignore a "No Results" list item
	nLine = pItem->text(1).toUInt();
	if (nLine == 0)
		return;
		
	// Emit the signal
	emit lineRequested(pItem->text(0), nLine);
	
	// Destroy the dialogue
	delete this;
}

/**
 * Creates a list item with record information returned by Cscope.
 * This slot is connected to the dataReady() signal emitted by the Cscope
 * process.
 * @param	pToken	Points to the head of the token list
 */
void QuickDefDlg::slotDataReady(FrontendToken* pToken)
{
	QString sFile, sLine, sText;
	
	// Get the file name
	sFile = pToken->getData();
	pToken = pToken->getNext();
	
	// Skip the function's name
	pToken = pToken->getNext();
	
	// Get the line number
	sLine = pToken->getData();
	pToken = pToken->getNext();
	
	// Get the line's text
	sText = pToken->getData();
	pToken = pToken->getNext();
	
	(void)new QListViewItem(m_pList, sFile, sLine, sText);
}

/**
 * Displays query progress information.
 * This slot is connected to the progress() signal emitted by the Cscope
 * process.
 * @param	nProgress	Progress value
 * @param	nTotal		The maximal progress value
 */
void QuickDefDlg::slotProgress(int nProgress, int nTotal)
{
	// Make sure the dialogue is visible for the progress information
	// Note that an nTotal value of 1 is used by default when the query
	// starts and ends, and does not indicate real progress
	if (nTotal > 1)
		show();
	
	// Display progress information
	m_progress.setProgress(nProgress, nTotal);
}

/**
 * Selects a record automatically, if only one record was produced by
 * Cscope, or displays the dialogue otherwise.
 * This slot is connected to the finished() signal emitted by the Cscope
 * process.
 * @param	nRecords	The total number of records generated
 */
void QuickDefDlg::slotFinished(uint nRecords)
{
	m_progress.finished();
	
	// If the query has resulted in a single record, notify the owner
	// immediately
	if (nRecords == 1) {
		slotRecordSelected(m_pList->firstChild());
		return;
	}
	
	// Make sure the dialogue is visible
	show();
	
	// Notify the user if no information was retrieved on this function
	if (nRecords == 0)
		(void)new QListViewItem(m_pList, i18n("No Results"));
}

#include "quickdefdlg.moc"

